"""校本化 Api"""
from typing import Dict, List, AnyStr

from yiban.Core import STask


class Task:
    def __init__(self, access_token):
        self.task = STask(access_token)

    @staticmethod
    def aes_encrypt(task_data) -> AnyStr:
        """提交表单加密"""
        return STask.aes_encrypt(task_data)

    @staticmethod
    def aes_decrypt(task_data) -> AnyStr:
        """提交表单解密"""
        return STask.aes_decrypt(task_data)

    def get_uncompleted_task(self) -> List[Dict]:
        """获取未完成任务"""
        return self.task.get_uncompleted_task()

    def get_completed_task(self) -> List[Dict]:
        """获取已完成任务"""
        return self.task.get_completed_task()

    def submit_task(self, task_data: dict) -> bool:
        """
        校本化 打卡提交
        :param task_data: （必须）提交表单
        :return:
        """
        return self.task.submit_task(task_data)



from base64 import b64encode
from typing import Dict, AnyStr

from Crypto.PublicKey import RSA
from Crypto.Cipher import PKCS1_v1_5


from yiban.Core import BaseYiban

"""
登录接口返回信息
{
    'response': 100, 
    'message': '请求成功', 
    'is_mock': False, 
    'data': {
        'access_token': '',
        'user': {
            'sex': '1', 
            'name': '姓名', 
            'nick': '昵称', 
            'user_id': int, 
            'phone': '电话号码', 
            'authority': '1', 
            'isSchoolVerify': True, 
            'school': {
                'isVerified': True, 
                'schoolName': '', 
                'schoolId': int, 
                'schoolOrgId': int, 
                'collegeName': '专业', 
                'collegeId': int, 
                'className': '', 
                'classId': 0, 
                'joinSchoolYear': '2020', 
                'type': 1
            }
        },
    }
}
"""


class Login(BaseYiban):
    def __init__(self):
        super().__init__()

    def get_user_info(self, mobile: str, password: str) -> Dict:
        """ Login function """
        # 暂时弃用
        # response = self.post(
        #     url="https://mobile.yiban.cn/api/v4/passport/login",
        #     headers={"User-Agent": "Yiban", "AppVersion": "5.0"},
        #     data={
        #         'ct': '2',
        #         'identify': '1',
        #         'mobile': mobile,
        #         'password': self.encrypt_rsa(password)}
        # ).json()

        # self._log(f"Login Response {response}", 10)

        # if response['response'] == 100:
        #     return response['data']

    def get_user_access_token(self, mobile: str, password: str) -> str:
        response = self.post(
            url="https://www.yiban.cn/login/doLoginAjax",
            headers={"User-Agent": "Yiban", "AppVersion": "5.0"},
            data={
                'account': mobile,
                'password': password
            }
        )
        self._log(f"Login Response {response}", 10)
        access_token = response.cookies.get_dict()['yiban_user_token']
        return access_token
        
    @staticmethod
    def encrypt_rsa(data: str) -> AnyStr:
        """
        登录密码加密
        :param data: （必须）待加密密码
        :return: Any
        """
        rsa_key = """-----BEGIN PUBLIC KEY-----
            MIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEA6aTDM8BhCS8O0wlx2KzA
            Ajffez4G4A/QSnn1ZDuvLRbKBHm0vVBtBhD03QUnnHXvqigsOOwr4onUeNljegIC
            XC9h5exLFidQVB58MBjItMA81YVlZKBY9zth1neHeRTWlFTCx+WasvbS0HuYpF8+
            KPl7LJPjtI4XAAOLBntQGnPwCX2Ff/LgwqkZbOrHHkN444iLmViCXxNUDUMUR9bP
            A9/I5kwfyZ/mM5m8+IPhSXZ0f2uw1WLov1P4aeKkaaKCf5eL3n7/2vgq7kw2qSmR
            AGBZzW45PsjOEvygXFOy2n7AXL9nHogDiMdbe4aY2VT70sl0ccc4uvVOvVBMinOp
            d2rEpX0/8YE0dRXxukrM7i+r6lWy1lSKbP+0tQxQHNa/Cjg5W3uU+W9YmNUFc1w/
            7QT4SZrnRBEo++Xf9D3YNaOCFZXhy63IpY4eTQCJFQcXdnRbTXEdC3CtWNd7SV/h
            mfJYekb3GEV+10xLOvpe/+tCTeCDpFDJP6UuzLXBBADL2oV3D56hYlOlscjBokNU
            AYYlWgfwA91NjDsWW9mwapm/eLs4FNyH0JcMFTWH9dnl8B7PCUra/Lg/IVv6HkFE
            uCL7hVXGMbw2BZuCIC2VG1ZQ6QD64X8g5zL+HDsusQDbEJV2ZtojalTIjpxMksbR
            ZRsH+P3+NNOZOEwUdjJUAx8CAwEAAQ==
            -----END PUBLIC KEY-----
            """
        data = bytes(data, encoding="utf8")
        encrypt = PKCS1_v1_5.new(RSA.importKey(rsa_key))
        sencrypt = b64encode(encrypt.encrypt(data))
        return sencrypt.decode("utf-8")



from re import findall
from json import dumps
from time import strftime, localtime
from base64 import b64encode, b64decode
from datetime import datetime, timedelta
from typing import Dict, List, AnyStr, Any

from Crypto.Cipher import AES

from yiban.Core import BaseYiban


class STask(BaseYiban):
    def __init__(self, access_token):
        super().__init__()
        self._csrf = '00000'
        self._access_token = access_token
        try:
            self._auth()
        except Exception as e:
            self._log(msg=f"STask Instantiation Failed. {e}", level=40)

    @staticmethod
    def _headers() -> Dict:
        return {"Origin": "https://c.uyiban.com", "User-Agent": "Yiban", "AppVersion": "5.0"}

    def _cookies(self) -> Dict:
        return {
            "yiban_user_token": self._access_token, 
            'loginToken': self._access_token, 
            'csrf_token': self._csrf
        }

    def _re_auth(self, verify: str) -> None:
        """
        校本化应用重认证
        :param verify: ...
        :return:
        """
        client_id = '95626fa3080300ea'
        redirect_uri = 'https://f.yiban.cn/iapp7463'
        self.get(
            url='https://api.uyiban.com/base/c/auth/yiban',
            params={'verifyRequest': verify, 'CSRF': self._csrf},
            headers=self._headers(),
            cookies=self._cookies()
        )

        self.get(
            url='https://oauth.yiban.cn/code/html',
            params={'client_id': client_id, 'redirect_uri': redirect_uri},
            headers=self._headers(),
            cookies=self._cookies()
        )

        self.post(
            url='https://oauth.yiban.cn/code/usersure',
            data={'client_id': client_id, 'redirect_uri': redirect_uri},
            headers=self._headers(),
            cookies=self._cookies()
        )

    def _auth(self) -> None:
        """用户认证"""
        "https://f.yiban.cn/iapp/index"
        "https://f.yiban.cn/iframe/index"

        response = self.get(
            # 认证接口 暂时替换
            url='https://f.yiban.cn/iframe/index',
            params={'act': 'iapp7463'},
            allow_redirects=False,
            headers=self._headers(),
            cookies=self._cookies()
        )
        verify = findall(r"verify_request=(.*?)&", response.headers.get("Location"))[0]

        self._re_auth(verify)

        response = self.get(
            url='https://api.uyiban.com/base/c/auth/yiban',
            params={'verifyRequest': verify, 'CSRF': self._csrf},
            headers=self._headers(),
            cookies=self._cookies()
        ).json()
        self._log(f"Auth Response {response}", 10)

        if response['code'] == 0:
            return None
        else:
            raise Exception(f"Auth Error {response['msg']}")

    def get_uncompleted_task(self) -> List[Dict]:
        """
        获取未完成任务
        :return: [
            {
                'TaskId': '',
                'OrgId': '',
                'TimeoutState': 1,
                'State': 0,
                'Title': '任务标题',
                'Type': 1,
                'StartTime': 1656172800,
                'EndTime': 1656230400
            }
        ]
        """
        response = self.get(
            url='https://api.uyiban.com/officeTask/client/index/uncompletedList',
            params={
                'StartTime': (datetime.now() + timedelta(days=-14)).strftime('%Y-%m-%d'),
                'EndTime': strftime("%Y-%m-%d 23:59", localtime()),
                'CSRF': self._csrf},
            headers=self._headers(),
            cookies=self._cookies()
        ).json()

        self._log(f"Get Uncompleted Tasks Response: {response}", 10)

        if response['code'] == 0:
            return response['data']
        else:
            raise Exception(f"Get Uncompleted Tasks Error {response['msg']}")

    def get_completed_task(self) -> List[Dict]:
        """获取已完成任务"""
        response = self.get(
            url='https://api.uyiban.com/officeTask/client/index/completedList',
            params={
                     'StartTime': (datetime.now() + timedelta(days=-14)).strftime('%Y-%m-%d'),
                     'EndTime': strftime("%Y-%m-%d 23:59", localtime()),
                     'CSRF': self._csrf},
            headers=self._headers(),
            cookies=self._cookies()
        ).json()

        self._log(f"Get Completed Tasks Response: {response}", 10)

        if response['code'] == 0:
            return response['data']
        else:
            raise Exception(f"Get Completed Tasks Error {response['msg']}")

    def get_task_id(self, task_title) -> AnyStr:
        """
        获取未完成任务中的 TaskId （以任务标题形式 ）
        :param task_title: （必须）任务标题
        :return:
        """
        for i in self.get_uncompleted_task():
            if i['Title'] == task_title:
                return i['TaskId']

        """未找到任务"""
        return None

    def get_task_wf_id(self, task_id) -> AnyStr:
        """
        获取任务 WFId
        :param task_id: （必须）任务ID
        :return: AnyStr | None
        """
        response = self.get(
            url='https://api.uyiban.com/officeTask/client/index/detail',
            params={'TaskId': task_id, 'CSRF': self._csrf},
            headers=self._headers(),
            cookies=self._cookies()
        ).json()

        self._log(f"Get Task WFId Response {response}", 10)

        if response['code'] == 0:
            return response["data"]["WFId"]
        else:
            raise Exception(f"Get Task WFId Error {response['msg']}")

    def submit_task(self, data: Dict) -> bool:
        """
        :param data: （必须）任务提交表单
        :return: bool Result
        :return: None 无未打卡任务
        """
        # 任务列表为空 无未完成任务
        if len(self.get_uncompleted_task()) == 0:
            self._log('All Tasks Completed')
            return None

        task_title = ''          # 任务标题
        task_data = data.copy()  # 为避免对实参修改

        for i in task_data['Extend']['content']:
            if i['label'] == '任务名称':
                task_title = i['value']

        task_id = self.get_task_id(task_title)     # 通过任务标题获取 TaskId

        # 未找到任务
        if task_id is None:
            self._log(f"Task Title '{task_title}' Not Found")
            return False

        task_wf_id = self.get_task_wf_id(task_id)  # 通过 TaskId 获取 WFId

        task_data['WFId'] = task_wf_id
        task_data['Extend']['TaskId'] = task_id
        task_data['Data'] = dumps(task_data['Data'], ensure_ascii=False)
        task_data['Extend'] = dumps(task_data['Extend'], ensure_ascii=False)
        task_data = self.aes_encrypt(dumps(task_data, ensure_ascii=False))

        # 提交表单
        response = self.post(
            url='https://api.uyiban.com/workFlow/c/my/apply',
            params={'CSRF': self._csrf},
            data={'Str': task_data},
            headers=self._headers(),
            cookies=self._cookies()
        ).json()

        self._log(f"Submit Task Response: {response}", 10)

        if response['code'] == 0:
            return True
        else:
            raise Exception(f"Submit Task Error {response['msg']}")

    @staticmethod
    def aes_encrypt(data: Any) -> AnyStr:
        """
        提交表单加密
        :param data: （必须）提交表单数据
        :return: AnyStr
        """
        aes_key = '2knV5VGRTScU7pOq'
        aes_iv = 'UmNWaNtM0PUdtFCs'
        aes_key = bytes(aes_key, 'utf-8')
        aes_iv = bytes(aes_iv, 'utf-8')
        data = bytes(data, 'utf-8')
        bs = AES.block_size
        padding = bs - len(data) % bs
        padding_text = bytes(chr(padding) * padding, 'utf-8')
        data = data + padding_text
        cipher = AES.new(aes_key, AES.MODE_CBC, aes_iv)
        encrypted = b64encode(cipher.encrypt(data))
        return b64encode(encrypted)

    @staticmethod
    def aes_decrypt(data: Any) -> AnyStr:
        """
        提交表单解密
        :param data: （必须）提交表单加密数据
        :return: AnyStr
        """
        aes_key = '2knV5VGRTScU7pOq'
        aes_iv = 'UmNWaNtM0PUdtFCs'
        aes_key = bytes(aes_key, 'utf-8')
        aes_iv = bytes(aes_iv, 'utf-8')
        data = b64decode(b64decode(data))
        cipher = AES.new(aes_key, AES.MODE_CBC, aes_iv)
        decrypted = cipher.decrypt(data)
        return decrypted.decode('Utf-8')


"""
任务信息接口
url = 'https://api.uyiban.com/officeTask/client/index/detail'
{
    'code': 0, 
    'msg': '', 
    'data': {
        'Id': '', 
        'Title': '', 
        'Type': 1, 'ReceiverSendBack': 0, 
        'TypeId': '', 
        'AllowSendBackHours': 0, 
        'AllowSendBackTimes': 1, 
        'PubOrgName': '',
        'PubPersonName': '', 
        'AllowTimeout': 1,
        'AttachmentIds': '[]', 
        'PubOrgId': '', 
        'IsPubPersonShow': 1, 
        'CreateTime': 1656172805, 
        'StartTime': 1656172800, 
        'EndTime': 1656230400, 
        'Displayed': 1, 
        'TimeState': 1, 
        'WFId': '', 
        'AttachmentList': [], 
        'WorkflowState': 0, 
        'InitiateId': '', 
        'RetreatReason': '', 
        'EvaluationState': 0, 
        'EvaluationReason': '', 
        'State': 'uncompleted', 
        'RecordState': 0, 
        'CanSendBack': 0, 
        'SendBackEndTime': 1656230400, 
        'IsLost': 0, 
        'Feedback': 0, 
        'ChangeReason': ''
    }
}
"""

"""
获取未完成任务接口信息
url = 'https://api.uyiban.com/officeTask/client/index/uncompletedList'
{
    'code': 0, 
    'msg': '', 
    'data': [
        {
            'TaskId': '', 
            'OrgId': '', 
            'TimeoutState': 1, 
            'State': 0, 
            'Title': '', 
            'Type': 1, 
            'StartTime': 0, 
            'EndTime': 0
        }
    ]
}
"""

"""
获取完成任务接口信息
url = 'https://api.uyiban.com/officeTask/client/index/completedList'
{
    'code': 0, 
    'msg': '', 
    'data': [
        {
            "TaskId": "",
            "State": 2,
            "OrgId": "",
            "FeedbackTime": ,
            "Title": "",
            "Type": 1,
            "StartTime": ,
            "EndTime": 
        }
    ]
}
"""

"""
提交表单接口信息
url = 'https://api.uyiban.com/workFlow/c/my/apply'
{
    'code': 0, 
    'msg': '', 
    'data': ''
}
"""



"""yiban"""
from .Apis import *

__title__ = 'yiban'
__description__ = 'Apis lib for yiban.'
__url__ = 'https://github.com/sricor/yiban'
__version__ = '1.0.3'
__author__ = 'joSricor'
__author_email__ = 'josricor@outlook.com'
__ide__ = 'PyCharm - https://www.jetbrains.com/pycharm/'



"""Yiban class"""

from yiban.Apis import User
from yiban.Apis import Task


class Yiban(
    User,
    Task,
):
    def __init__(
            self,
            mobile:   str,
            password: str,
    ):
        """
        易班 Python API service
        :param mobile:   (必须) 用户账号
        :param password: (必须) 用户密码
        """
        super().__init__(mobile=mobile, password=password)  # init User
        super(User, self).__init__(access_token=self.get_user_access_token())  # init Task







from yiban.Core import Login


class User:
    def __init__(self, mobile: str, password: str):
        self.__mobile = mobile
        self.__password = password
        self.user = Login()

    def get_user_access_token(self) -> str:
        """获取用户登录密钥"""
        return self.user.get_user_access_token(self.__mobile, self.__password)

    def get_user_name(self) -> str:
        """获取用户名"""
        return self.user['user']['name']

    def get_user_phone(self) -> str:
        """获取用户手机号"""
        return self.user['user']['phone']

    def get_user_school_name(self) -> str:
        """获取用户学校名称"""
        return self.user['user']['school']['schoolName']



from logging import log
from typing import Dict

import requests


class BaseYiban:
    def __init__(self):
        self.session = requests.session()

    @staticmethod
    def _log(msg: str = None, level: int = 20) -> None:
        msg = f"Yiban {msg}"
        log(msg=msg, level=level)

    def request(
            self, 
            method:  str, 
            url:     str, 
            params:  Dict = None,
            headers: Dict = None,
            cookies: Dict = None,
            data:    Dict = None, 
            body:    Dict = None,
            allow_redirects: bool = True
    ) -> requests.Response:
        
        """统一请求方法"""
        # 删除值为None的键
        if body is not None:
            body = {k: v for k, v in body.items() if v is not None}

        if data is not None and isinstance(data, dict):
            data = {k: v for k, v in data.items() if v is not None}

        for i in range(1, 6):
            response = self.session.request(method=method, url=url, params=params, 
                                            data=data, headers=headers, cookies=cookies,
                                            json=body, allow_redirects=allow_redirects, timeout=20)

            return response

        # 重试五次失败
        self._log(f"Request Error {url}")
        raise Exception("Request Error")
        
    def get(self, url: str, params: dict = None, headers: dict = None,
            cookies: dict = None, allow_redirects: bool = True) -> requests.Response:
        """..."""
        return self.request(method='GET', url=url, params=params, headers=headers,
                            cookies=cookies, allow_redirects=allow_redirects)

    def post(self, url: str, params: dict = None, headers: dict = None,
             data: dict = None, cookies: dict = None,
             body: dict = None, allow_redirects: bool = True) -> requests.Response:
        """..."""
        return self.request(method='POST', url=url, params=params,
                            data=data, headers=headers, cookies=cookies,
                            body=body, allow_redirects=allow_redirects)
